#include <cerrno>
#include "lock_file.h"

lock_file::lock_file(const std::string& filename)
  : _fd(-1),
    _lock_state(UNLOCKED),
    _filename(filename)
{
  // Initialize structure used to lock the file.
  _lock_flock.l_type = F_WRLCK;
  _lock_flock.l_whence = SEEK_SET;
  _lock_flock.l_start = 0;
  _lock_flock.l_len = 0;

  // Initialize structure used to unlock the file.
  _unlock_flock.l_type = F_WRLCK;
  _unlock_flock.l_whence = SEEK_SET;
  _unlock_flock.l_start = 0;
  _unlock_flock.l_len = 0;

  // Get the file descriptor creating the file if necessary.
  _fd = xunix::open(_filename.c_str(),
                    O_RDWR | O_CREAT,
                    S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
}

lock_file::~lock_file()
{
  xunix::close(_fd);
}

// Be careful with lock() because a signal could interrupt our
// waiting.  Sometimes that is what you want, and sometimes it isn't.
// If it isn't what you want, use a try block to restart the lock()
// call.
void lock_file::lock()
{
  if( _lock_state == UNLOCKED ) {
    xunix::fcntl(_fd, F_SETLKW, &_lock_flock);
    _lock_state = LOCKED;
  }
}

void lock_file::unlock()
{
  if( _lock_state == LOCKED ) {
    xunix::fcntl(_fd, F_SETLK, &_unlock_flock);
    _lock_state = UNLOCKED;
  }
}
